#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <X11/Xlib.h>
#include <Imlib.h>
#include "sbig.h"

struct {
    Display  *display;
    ImlibData *im;
    ImlibImage *image;
    Window  window;
} x_info;
struct sbig_init  info;

#define RED 0
#define GREEN 1
#define BLUE 2

void load_image(unsigned short *sp, int w, int h)
{
    unsigned char  *dp;
    double  f;
    int  d, i, j, min, max;

    if (x_info.image != NULL && (w != x_info.image->rgb_width != w ||
	  h != x_info.image->rgb_height)) {
	Imlib_destroy_image(x_info.im, x_info.image);
	x_info.image = NULL;
    }
    if (x_info.image == NULL) {
	unsigned char *data = malloc(w*h*3);
	x_info.image = Imlib_create_image_from_data(x_info.im,
	  data, NULL, w, h);
	free(data);
	if (x_info.image == NULL)
	    return;
    }
    /* get min and max */
    min = 65535;
    max = 0;
    for (i = w*h; --i >= 0; ) {
	if (sp[i] > max)
	    max = sp[i];
	else if (sp[i] < min)
	    min = sp[i];
    }

    dp = x_info.image->rgb_data;
    if (max == min)
	min = max - 1;
    for (j = 0; j < h; ++j) {
	for (i = 0; i < w; ++i) {
	    f = ((double)*sp++ - min)/(max - min);
	    d = (int)(255.0*pow(f, 0.33));	/* gamma correct */
	    if (d < 0) {
		printf("d = %d at %d,%d\n", d, i, j);
		d = 0;
	    } else if (d > 255) {
		printf("d = %d at %d,%d\n", d, i, j);
		d = 255;
	    }
	    dp[RED] = d;
	    dp[GREEN] = d;
	    dp[BLUE] = d;
	    if (i == j) {
		dp[RED]=255;
		dp[GREEN]=0;
		dp[BLUE]=0;
	    }
	    dp += 3;
	}
    }
    XResizeWindow(x_info.display, x_info.window, w, h);
    Imlib_changed_image(x_info.im, x_info.image);
    Imlib_paste_image(x_info.im, x_info.image, x_info.window, 0, 0, w, h);
    XFlush(x_info.display);
}

int geti(char *s, int default_value)
{
    char buffer[80];

    printf(s);
    fflush(stdout);
    fgets(buffer, 79, stdin);
    if (buffer[0] == '\n')
	return default_value;
    return atoi(buffer);
}

double getf(char *s, float default_value)
{
    char buffer[80];

    printf(s);
    fflush(stdout);
    fgets(buffer, 79, stdin);
    if (buffer[0] == '\n')
	return default_value;
    return atof(buffer);
}

void print_stats(unsigned short *b, int w, int h)
{
    int	 min = 65536, max = -1;
    int  i_min = 0, i_max = 0, i;

    for (i = w*h; --i >= 0; ) {
	if ((int)b[i] < min) {
	    i_min = i;
	    min = b[i];
	}
	if ((int)b[i] > max) {
	    i_max = i;
	    max = b[i];
	}
    }
    printf("min=%d at (%d,%d); ", min, i_min%w, i_min/w);
    printf("max=%d at (%d,%d)\n", max, i_max%w, i_max/w);
}

/*
 * This is the 'progress' callback
 * passed to sbig_readout()
 */
int snap_callback(float percent)
{
    printf("\r%4.1f%% download complete ", percent);
    fflush(stdout);
    /* return 1 to continue, 0 to abort */
    return 1;
}

void snap(void)
{
    int  ret, i, needed;
    char  input[80];
    static unsigned short  *buffer = NULL, *dark_buffer = NULL;
    static int  buffer_size;
    struct sbig_expose  expose;
    struct sbig_status  status;
    struct sbig_readout readout;
    double  f;

    /* loop until we get good input */
    while (1) {
	expose.ccd = geti("ccd? [*0=imaging, 1=tracking] ", 0);
	if (expose.ccd == 0)
	    readout.binning = geti("Binning mode [0=1x1, *1=2x2, 2=3x3]: ", 1);
	else if (expose.ccd == 1)
	    readout.binning = geti("Binning mode [0=1x1, *1=2x2]: ", 1);
	else {
	    printf("Invalid ccd %d\n", expose.ccd);
	    continue;
	}
	if (readout.binning < 0 || readout.binning > (expose.ccd == 0? 2:1)) {
	    printf("Invalid binning mode %d\n", readout.binning);
	    continue;
	}
	break;
    }

    /* loop until we get good input */
    while (1) {
	printf("Enter x, y, w, h for image [CR for default]: ");
	fflush(stdout);
	fgets(input, 79, stdin);
	if (input[0] == '\n') {
	    readout.x = readout.y = 0;
	    readout.width =
	      info.camera_info[expose.ccd].readout_mode[readout.binning].width;
	    readout.height =
	      info.camera_info[expose.ccd].readout_mode[readout.binning].height;
	    printf("image size is %d x %d\n", readout.width, readout.height);
	} else
	    sscanf(input, "%d %d %d %d", &readout.x, &readout.y,
		   &readout.width, &readout.height);
	if (readout.x < 0 || readout.y < 0) {
	    printf("invalid starting coordinate %d,%d\n",
		    readout.x, readout.y);
	    continue;
	}
	if (readout.width <= 0 || readout.height <= 0) {
	    printf("invalid width or height %dx%d\n", readout.width, readout.height);
	    continue;
	}
	break;
    }

    f = getf("Enter exposure time (sec) [default 5.0]: ", 5.0);
    expose.exposure_time = (int)(100.0*f);
    expose.abg_state = 0;

    /*
     * take a dark
     */
    expose.shutter = 0;
    sync();
    ret = sbig_expose(&expose);
    if (ret < 0) {
	printf("sbig error: %s\n", sbig_show_error(ret));
	return;
    }
    if (expose.exposure_time > 100) {
	for (i = 0; i < expose.exposure_time; i += 100) {
	    sbig_get_status(&status);
	    printf("%4.1f%% ", 100.0*i/expose.exposure_time);
	    printf("imaging_ccd_status=%d, shutter_state=%d\r",
		status.imaging_ccd_status, status.shutter_state);
	    fflush(stdout);
	    sleep(1);
	}
    } else
	usleep(expose.exposure_time*10000);

    /* wait for exposure to complete */
    for (i = 0; i < 1000; ++i) {
	ret = sbig_get_status(&status);
	if (ret < 0) {
	    printf("sbig error: %s\n", sbig_show_error(ret));
	    break;
	}
	if (readout.ccd == 0) {
	    if (status.imaging_ccd_status != 2)
		break;
	} else {
	    if (status.tracking_ccd_status != 2)
		break;
	}
	usleep(50000);
    }
    if (i == 1000)
	printf("EXPOSURE DIDN'T FINISH!\n");
    printf("100%%  imaging_ccd_status=%d, shutter_state=%d\n",
		status.imaging_ccd_status, status.shutter_state);

    readout.ccd = expose.ccd;
    readout.binning = readout.binning;
    needed = readout.height*readout.width*sizeof(short);
    if (buffer_size < needed) {
	if (dark_buffer != NULL)
	    free(dark_buffer);
	if (buffer != NULL)
	    free(buffer);
	dark_buffer = malloc(buffer_size = needed);
	buffer = malloc(buffer_size);
    }
    readout.data = dark_buffer;
    readout.data_size_in_bytes = buffer_size;
    readout.callback = snap_callback;

    sync();
    if ((ret = sbig_readout(&readout)) < 0) {
	printf("sbig error: %s\n", sbig_show_error(ret));
	return;
    }
    printf("\ndark frame: ");
    print_stats(dark_buffer, readout.width, readout.height);

    /*
     * take a frame
     */
    expose.shutter = 1;
    ret = sbig_expose(&expose);
    if (ret < 0) {
	printf("sbig error: %s\n", sbig_show_error(ret));
	return;
    }
    if (expose.exposure_time > 100) {
	for (i = 0; i < expose.exposure_time; i += 100) {
	    sbig_get_status(&status);
	    printf("%4.1f%% ", 100.0*i/expose.exposure_time);
	    printf("imaging_ccd_status=%d, shutter_state=%d\r",
		status.imaging_ccd_status, status.shutter_state);
	    fflush(stdout);
	    sleep(1);
	}
    } else
	usleep(expose.exposure_time*10000);

    /* wait for exposure to complete */
    for (i = 0; i < 1000; ++i) {
	ret = sbig_get_status(&status);
	if (ret < 0) {
	    printf("sbig error: %s\n", sbig_show_error(ret));
	    break;
	}
	if (readout.ccd == 0) {
	    if (status.imaging_ccd_status != 2)
		break;
	} else {
	    if (status.tracking_ccd_status != 2)
		break;
	}
	usleep(50000);
    }
    if (i == 1000)
	printf("EXPOSURE DIDN'T FINISH!\n");
    printf("100%%  imaging_ccd_status=%d, shutter_state=%d\n",
		status.imaging_ccd_status, status.shutter_state);

    sync();
    readout.data = buffer;
    if ((ret = sbig_readout(&readout)) < 0) {
	printf("sbig error: %s\n", sbig_show_error(ret));
	return;
    }
    printf("\nimage: ");
    print_stats(buffer, readout.width, readout.height);

    /* subtract out the dark, and adjust the image */
    for (i = readout.width*readout.height; --i >= 0; ) {
	int  d;
	d = buffer[i] - dark_buffer[i];
	if (d < 0)
	    d = 0;
	else if (d > 65535)
	    d = 65535;
	buffer[i] = d;
    }
    load_image(buffer, readout.width, readout.height);
}

int main(int argc, char *argv[])
{
    int  ret, i, port;
    fd_set  read_fds;

    port = 1; /* 0x378 */
    for (i = 1; i < argc; ++i) {
	if (!strcmp(argv[i], "-port")) {
	    sscanf(argv[++i], "%x", &port);
	    printf("port override to 0x%x\n", port);
	}
    }

    for (i = 0; i < 4; ++i) {
	sync();
	/* set our verbosity to 2 (range is 0 to 5) */
	ret = sbig_init(port, 2, &info);
	printf("sbig_init() returned %d\n", ret);
	if (ret < 0)
	    printf("sbig error: %s\n", sbig_show_error(ret));
	else
	    break;
	sleep(1);
    }
    if (ret < 0) {
	printf("cannot open camera, giving up\n");
	exit(1);
    }
    printf("linux_version: %g\n", info.linux_version);
    printf("firmware_version: %u\n", info.firmware_version);
    printf("Imaging ABG: %s\n", info.imaging_abg_type? "Present":"None");
    printf("Imaging bad columns: %d\n", info.nmbr_bad_columns);
    for (i = 0; i < info.nmbr_bad_columns; ++i)
	printf("  column %d\n", info.bad_columns[i]);
    printf("Serial Number: %s\n", info.serial_number);
    printf("Camera name: %s\n", info.camera_name);
    for (i = 0; i < info.camera_info[0].nmbr_readout_modes; ++i) {
	struct readout_mode  *rmp;
	rmp = &info.camera_info[0].readout_mode[i];
	printf("ImagingCCD mode %d: w=%d h=%d ", rmp->mode, rmp->width,
		rmp->height);
	printf("gain=%f ", 0.01*rmp->gain);
	printf("pw=%g ph=%g\n", 0.001*rmp->pixel_width, 0.001*rmp->pixel_height);
    }
    for (i = 0; i < info.camera_info[1].nmbr_readout_modes; ++i) {
	struct readout_mode  *rmp;
	rmp = &info.camera_info[1].readout_mode[i];
	printf("TrackingCCD mode %d: w=%d h=%d ", rmp->mode, rmp->width,
		rmp->height);
	printf("gain=%f ", 0.01*rmp->gain);
	printf("pw=%g ph=%g\n", 0.001*rmp->pixel_width, 0.001*rmp->pixel_height);
    }

    /*
     * set up X11
     */
    if ((x_info.display = XOpenDisplay(NULL)) == NULL) {
	printf("cannot open display\n");
	exit(1);
    }
    x_info.im = Imlib_init(x_info.display);
    x_info.window = XCreateWindow(x_info.display,
	DefaultRootWindow(x_info.display), 0, 0, 100, 100, 0,
	x_info.im->x.depth, InputOutput,
	x_info.im->x.visual, 0, NULL);
    XSelectInput(x_info.display, x_info.window, ExposureMask);
    XMapWindow(x_info.display, x_info.window);
    x_info.image = NULL;

    printf("To expose an image, press ENTER\n");
    while (1) {
	FD_ZERO(&read_fds);
	FD_SET(0, &read_fds);
	FD_SET(ConnectionNumber(x_info.display), &read_fds);
	select(32, &read_fds, NULL, NULL, NULL);
	if (FD_ISSET(ConnectionNumber(x_info.display), &read_fds)) {
	    XEvent ev;
	    XNextEvent(x_info.display, &ev);
	    if (ev.type==Expose) {
		if (ev.xexpose.count == 0 && x_info.image != NULL) {
		    printf("Imlib_paste_image\n");
	            Imlib_paste_image(x_info.im, x_info.image, x_info.window,
		      0, 0, x_info.image->rgb_width, x_info.image->rgb_height);
		    XFlush(x_info.display);
		}
	    }
	}
	if (FD_ISSET(0, &read_fds)) {
	    getchar();  /* suck up ENTER */
	    snap();
	    printf("To expose an image, press ENTER\n");
	}
    }
}
